@@ -1,38 +1,42 @@
-import logging
 import os
 import pathlib
 import re
 import uuid
+import warnings
 from inspect import isclass
 
-import ipywidgets as w
+import ipyvue
 
 from astropy.nddata import CCDData
 from echo import CallbackProperty, DictCallbackProperty, ListCallbackProperty
 from ipygoldenlayout import GoldenLayout
 from ipysplitpanes import SplitPanes
-from traitlets import Dict, Bool
-from regions import RectanglePixelRegion, PixCoord
-from specutils import Spectrum1D
+from traitlets import Dict, Bool, Unicode
+from specutils import Spectrum1D, SpectralRegion
+import numpy as np
 
+from glue.core.exceptions import IncompatibleAttribute
 from glue.config import data_translator
+from glue.config import settings as glue_settings
 from glue.core import BaseData, HubListener, Data, DataCollection
 from glue.core.link_helpers import LinkSame
 from glue.core.message import (DataCollectionAddMessage,
                                DataCollectionDeleteMessage)
 from glue.core.state_objects import State
-from glue.core.subset import Subset
+from glue.core.subset import Subset, RangeSubsetState, RoiSubsetState
 from glue_jupyter.app import JupyterApplication
 from glue_jupyter.state_traitlets_helpers import GlueState
+from glue_jupyter.bqplot.profile import BqplotProfileView
 from ipyvuetify import VuetifyTemplate
 
-from .core.config import read_configuration, get_configuration
-from .core.events import (LoadDataMessage, NewViewerMessage, AddDataMessage,
-                          SnackbarMessage, RemoveDataMessage,
-                          AddDataToViewerMessage, RemoveDataFromViewerMessage)
-from .core.registries import (tool_registry, tray_registry, viewer_registry,
-                              data_parser_registry)
-from .utils import load_template
+from jdaviz.core.config import read_configuration, get_configuration
+from jdaviz.core.events import (LoadDataMessage, NewViewerMessage, AddDataMessage,
+                                SnackbarMessage, RemoveDataMessage,
+                                AddDataToViewerMessage, RemoveDataFromViewerMessage,
+                                ViewerAddedMessage, ViewerRemovedMessage)
+from jdaviz.core.registries import (tool_registry, tray_registry, viewer_registry,
+                                    data_parser_registry)
+from jdaviz.utils import SnackbarQueue
 
 __all__ = ['Application']
 
@@ -45,6 +49,39 @@ EXT_TYPES = dict(flux=['flux', 'sci'],
                  mask=['mask', 'dq'])
 
 
+# Set default opacity for data layers to 1 instead of 0.8 in
+# some glue-core versions
+glue_settings.DATA_ALPHA = 1
+
+ipyvue.register_component_from_file(None, 'j-tooltip',
+                                    os.path.join(os.path.dirname(__file__),
+                                                 'components/tooltip.vue'))
+
+ipyvue.register_component_from_file(None, 'j-external-link',
+                                    os.path.join(os.path.dirname(__file__),
+                                                 'components/external_link.vue'))
+
+ipyvue.register_component_from_file(None, 'j-docs-link',
+                                    os.path.join(os.path.dirname(__file__),
+                                                 'components/docs_link.vue'))
+
+ipyvue.register_component_from_file(None, 'j-tray-plugin',
+                                    os.path.join(os.path.dirname(__file__),
+                                                 'components/tray_plugin.vue'))
+
+ipyvue.register_component_from_file(None, 'j-plugin-section-header',
+                                    os.path.join(os.path.dirname(__file__),
+                                                 'components/plugin_section_header.vue'))
+
+# Register pure vue component. This allows us to do recursive component instantiation only in the
+# vue component file
+ipyvue.register_component_from_file('g-viewer-tab', "container.vue", __file__)
+
+ipyvue.register_component_from_file(None, 'j-play-pause-widget',
+                                    os.path.join(os.path.dirname(__file__),
+                                                 'components/play_pause_widget.vue'))
+
+
 class ApplicationState(State):
     """
     The application state object contains all the current front-end state,
@@ -66,6 +103,8 @@ class ApplicationState(State):
         'loading': False
     }, docstring="State of the quick toast messages.")
 
+    snackbar_queue = SnackbarQueue()
+
     settings = DictCallbackProperty({
         'data': {
             'auto_populate': False,
@@ -77,6 +116,7 @@ class ApplicationState(State):
             'tray': True,
             'tab_headers': True,
         },
+        'dense_toolbar': True,
         'context': {
             'notebook': {
                 'max_height': '600px'
@@ -95,6 +135,9 @@ class ApplicationState(State):
     tray_items = ListCallbackProperty(
         docstring="List of plugins displayed in the sidebar tray area.")
 
+    tray_items_open = CallbackProperty(
+        [], docstring="The plugin(s) opened in sidebar tray area.")
+
     stack_items = ListCallbackProperty(
         docstring="Nested collection of viewers constructed to support the "
                   "Golden Layout viewer area.")
@@ -109,19 +152,14 @@ class Application(VuetifyTemplate, HubListener):
 
     state = GlueState().tag(sync=True)
 
-    template = load_template("app.vue", __file__).tag(sync=True)
-
-    # Pure vue components are added through the components attribute. This
-    #  allows us to do recursive component instantiation only in the vue
-    #  component file
-    components = Dict({"g-viewer-tab": load_template(
-        "container.vue", __file__, traitlet=False)}).tag(
-            sync=True, **w.widget_serialization)
+    template_file = __file__, "app.vue"
 
     loading = Bool(False).tag(sync=True)
+    config = Unicode("").tag(sync=True)
 
     def __init__(self, configuration=None, *args, **kwargs):
         super().__init__(*args, **kwargs)
+        self._verbosity = 'info'
 
         # Generate a state object for this application to maintain the state of
         #  the user interface.
@@ -129,7 +167,13 @@ class Application(VuetifyTemplate, HubListener):
 
         # The application handler stores the state of the data and the
         #  underlying glue infrastructure
-        self._application_handler = JupyterApplication()
+        self._application_handler = JupyterApplication(
+            settings={'new_subset_on_selection_tool_change': True})
+
+        # Add a reference to this application to the Glue session object. This
+        # allows the jdaviz Application object to then be accessed via e.g.
+        # viewer.session.jdaviz_app
+        self._application_handler.session.jdaviz_app = self
 
         # Create a dictionary for holding non-ipywidget viewer objects so we
         #  can reference their state easily since glue does not store viewers
@@ -138,6 +182,10 @@ class Application(VuetifyTemplate, HubListener):
         # Parse the yaml configuration file used to compose the front-end UI
         self.load_configuration(configuration)
 
+        # If true, link data on load. If false, do not link data to speed up
+        # data loading
+        self.auto_link = kwargs.pop('auto_link', True)
+
         # Subscribe to messages indicating that a new viewer needs to be
         #  created. When received, information is passed to the application
         #  handler to generate the appropriate viewer instance.
@@ -181,57 +229,72 @@ class Application(VuetifyTemplate, HubListener):
     @property
     def hub(self):
         """
-        Reference to the stored application handler hub instance.
-
-        Returns
-        -------
-        `glue.core.Hub`
-            The internal ``Hub`` instance for the application.
+        Reference to the stored application handler `~glue.core.hub.Hub` instance
+        for the application.
         """
         return self._application_handler.data_collection.hub
 
     @property
     def session(self):
         """
-        Reference to the stored session instance.
-
-        Returns
-        -------
-        `glue.core.Session`
-            The ``Session`` instance maintained by Glue for this application.
+        Reference to the stored `~glue.core.session.Session` instance
+        maintained by Glue for this application.
         """
         return self._application_handler.session
 
     @property
     def data_collection(self):
         """
-        Reference to the stored data collection instance, used to maintain the
-        the data objects that been loaded into the application this session.
-
-        Returns
-        -------
-        `glue.core.DataCollection`
-            The ``DataCollection`` instance for this application session.
+        Reference to the stored `~glue.core.data_collection.DataCollection` instance,
+        used to maintain the the data objects that been loaded into the application
+        this session.
         """
         return self._application_handler.data_collection
 
+    @property
+    def verbosity(self):
+        """
+        Verbosity of the application, choose from ``'debug'``, ``'info'`` (default),
+        ``'warning'``, or ``'error'``.
+        """
+        return self._verbosity
+
+    @verbosity.setter
+    def verbosity(self, val):
+        if val not in ('debug', 'info', 'warning', 'error'):
+            raise ValueError(f'Invalid verbosity: {val}')
+        self._verbosity = val
+
     def _on_snackbar_message(self, msg):
         """
         Displays a toast message with an editable message that be dismissed
         manually or will dismiss automatically after a timeout.
 
+        This is also controlled by ``self.verbosity``.
+
         Parameters
         ----------
         msg : `~glue.core.SnackbarMessage`
             The Glue snackbar message containing information about displaying
             the message box.
         """
-        self.state.snackbar['show'] = False
-        self.state.snackbar['text'] = msg.text
-        self.state.snackbar['color'] = msg.color
-        self.state.snackbar['timeout'] = msg.timeout
-        self.state.snackbar['loading'] = msg.loading
-        self.state.snackbar['show'] = True
+        # https://material-ui.com/customization/palette/ provides these options:
+        #   success, info, warning, error, secondary, primary
+        # We have these options:
+        #   debug, info, warning, error
+        # Therefore:
+        # * debug is not used, it is more for the future if we also have a logger.
+        # * info lets everything through
+        # * success, secondary, and primary are treated as info (not sure what they are used for)
+        # * None is also treated as info (when color is not set)
+        ignored = []
+        if self.verbosity in ('warning', 'error'):
+            ignored += ['success', 'info', 'secondary', 'primary', None]
+            if self.verbosity == 'error':
+                ignored.append('warning')
+
+        if msg.color not in ignored:
+            self.state.snackbar_queue.put(self.state, msg)
 
     def _link_new_data(self):
         """
@@ -240,13 +303,19 @@ class Application(VuetifyTemplate, HubListener):
         them so that they can be displayed on the same profile1D plot.
         """
         new_len = len(self.data_collection)
+
+        # Allow for batch linking of data in the parser rather than on
+        # data load
+        if not self.auto_link or new_len < 2:
+            return
+
         # Can't link if there's no world_component_ids
-        wc_new = self.data_collection[new_len-1].world_component_ids
+        wc_new = self.data_collection[-1].world_component_ids
         if wc_new == []:
             return
 
         # Link to the first dataset with compatible coordinates
-        for i in range(0, new_len-1):
+        for i in range(new_len - 1):
             wc_old = self.data_collection[i].world_component_ids
             if wc_old == []:
                 continue
@@ -257,7 +326,7 @@ class Application(VuetifyTemplate, HubListener):
     def load_data(self, file_obj, parser_reference=None, **kwargs):
         """
         Provided a path to a data file, open and parse the data into the
-        `~glue.core.DataCollection` for this session. This also attempts to
+        `~glue.core.data_collection.DataCollection` for this session. This also attempts to
         find WCS links that exist between data components. Extra key word
         arguments are passed to the parsing functions.
 
@@ -308,10 +377,11 @@ class Application(VuetifyTemplate, HubListener):
             else:
                 self._application_handler.load_data(file_obj)
 
-            # Send out a toast message
-            snackbar_message = SnackbarMessage("Data successfully loaded.",
-                                               sender=self)
-            self.hub.broadcast(snackbar_message)
+        except Exception:  # Reset state on uncaught errors
+            cfg_name = self.state.settings.get('configuration', 'unknown')
+            if cfg_name in ('mosviz', ):  # Add more as needed.
+                self.data_collection.clear()
+            raise
         finally:
             self.loading = False
 
@@ -342,6 +412,12 @@ class Application(VuetifyTemplate, HubListener):
         """
         return self._viewer_by_reference(viewer_reference)
 
+    def get_viewer_by_id(self, vid):
+        """Like :meth:`get_viewer` but use ID instead of reference name.
+        This is useful when reference name is `None`.
+        """
+        return self._viewer_store[vid]
+
     def get_data_from_viewer(self, viewer_reference, data_label=None,
                              cls='default', include_subsets=True):
         """
@@ -431,8 +507,11 @@ class Application(VuetifyTemplate, HubListener):
 
                         if cls is not None:
                             handler, _ = data_translator.get_handler_for(cls)
-                            layer_data = handler.to_object(layer_data,
-                                                           statistic=statistic)
+                            try:
+                                layer_data = handler.to_object(layer_data,
+                                                               statistic=statistic)
+                            except IncompatibleAttribute:
+                                continue
 
                         data[label] = layer_data
 
@@ -442,7 +521,7 @@ class Application(VuetifyTemplate, HubListener):
 
         return data
 
-    def get_subsets_from_viewer(self, viewer_reference, data_label=None):
+    def get_subsets_from_viewer(self, viewer_reference, data_label=None, subset_type=None):
         """
         Returns the subsets of a specified viewer converted to astropy regions
         objects.
@@ -463,6 +542,10 @@ class Application(VuetifyTemplate, HubListener):
         data_label : str, optional
             Optionally provide a label to retrieve a specific data set from the
             viewer instance.
+        subset_type : str, optional
+            Optionally specify either "spectral" or "spatial" to return only
+            subsets created in a profile (spectrum) viewer or image viewer,
+            respectively.
 
         Returns
         -------
@@ -477,17 +560,83 @@ class Application(VuetifyTemplate, HubListener):
                                          cls=None)
         regions = {}
 
+        def _get_all_subregions(mask, spec_axis_data):
+            """
+            Return all subregions within a subset.
+
+            Parameters
+            ----------
+            mask : list
+                List of indices in spec_axis_data that are part of the subset.
+            spec_axis_data : list
+                List of spectral axis values.
+            Returns
+            -------
+            combined_spec_region : `~specutils.SpectralRegion`
+                SpectralRegion object containing all subregions of the subset.
+            """
+            if len(mask) == 0:
+                # Mask should only be 0 if ApplyROI is used to incorrectly
+                # create subsets via the API
+                raise ValueError("Mask has length 0, ApplyROI may have been used incorrectly")
+
+            current_edge = 0
+            combined_spec_region = None
+            for index in range(1, len(mask)):
+                # Find spot where mask == True is for a different region of the subset
+                # i.e. mask = [0, 1, 4, 5]
+                # mask[2] != mask[1] + 1
+                if mask[index] != mask[index - 1] + 1:
+                    subset_region = spec_axis_data[mask[current_edge]: mask[index - 1] + 1]
+                    if not combined_spec_region:
+                        combined_spec_region = SpectralRegion(min(subset_region),
+                                                              max(subset_region))
+                    else:
+                        combined_spec_region += SpectralRegion(min(subset_region),
+                                                               max(subset_region))
+                    current_edge = index
+
+            # Get last region within the subset
+            if current_edge != index:
+                subset_region = spec_axis_data[mask[current_edge]: mask[index] + 1]
+                # No if check here because len(mask) must be greater than 1
+                # so combined_spec_region will have been instantiated in the for loop
+                if combined_spec_region is None:
+                    combined_spec_region = SpectralRegion(min(subset_region), max(subset_region))
+                else:
+                    combined_spec_region += SpectralRegion(min(subset_region), max(subset_region))
+
+            return combined_spec_region
+
+        if data_label is not None:
+            data = {data_label: data}
+
         for key, value in data.items():
             if isinstance(value, Subset):
+                # Get the component type in a compound subset
+                if hasattr(value.subset_state, "state1"):
+                    this_type = type(value.subset_state.state1)
+                else:
+                    this_type = type(value.subset_state)
+
+                # Skip spatial or spectral subsets if only the other is wanted
+                if subset_type == "spectral" or isinstance(viewer, BqplotProfileView):
+                    if this_type == RoiSubsetState:
+                        continue
+                elif subset_type == "spatial" or not isinstance(viewer, BqplotProfileView):
+                    if this_type == RangeSubsetState:
+                        continue
+
                 # Range selection on a profile is currently not supported in
                 #  the glue translation machinery for astropy regions, so we
                 #  have to do it manually. Only data that is 2d is supported,
                 #  therefore, if the data is already 2d, simply use as is.
                 if value.data.ndim == 2:
                     region = value.data.get_selection_definition(
-                        format='astropy-regions')
+                        subset_id=key, format='astropy-regions')
                     regions[key] = region
                     continue
+
                 # There is a special case for 1d data (which is also not
                 #  supported currently). We now eschew the use of the
                 #  translation machinery entirely and construct the astropy
@@ -496,14 +645,22 @@ class Application(VuetifyTemplate, HubListener):
                     # Grab the data units from the glue-astronomy spectral axis
                     # TODO: this needs to be much simpler; i.e. data units in
                     #  the glue component objects
-                    unit = value.data.coords.spectral_axis.unit
-                    hi, lo = value.subset_state.hi, value.subset_state.lo
-                    xcen = 0.5 * (lo + hi)
-                    width = hi - lo
-                    region = RectanglePixelRegion(
-                        PixCoord(xcen, 0), width, 0,
-                        meta={'spectral_axis_unit': unit})
-                    regions[key] = region
+                    # Cases where there is a single subset
+                    subregions_in_subset = _get_all_subregions(
+                            np.where(value.to_mask() == True)[0], # noqa
+                            value.data.coords.spectral_axis)
+
+                    regions[key] = subregions_in_subset
+                    continue
+
+                temp_data = self.get_data_from_viewer(viewer_reference, value.label)
+                if isinstance(temp_data, Spectrum1D):
+                    # Note that we look for mask == False here, rather than True above,
+                    # because specutils masks are the reverse of Glue (of course)
+                    subregions_in_subset = _get_all_subregions(
+                             np.where(~temp_data.mask)[0], # noqa
+                             temp_data.spectral_axis)
+                    regions[key] = subregions_in_subset
                     continue
 
                 # Get the pixel coordinate [z] of the 3D data, repeating the
@@ -556,7 +713,7 @@ class Application(VuetifyTemplate, HubListener):
 
         return regions
 
-    def add_data(self, data, data_label):
+    def add_data(self, data, data_label=None, notify_done=True):
         """
         Add data to the Glue ``DataCollection``.
 
@@ -568,18 +725,25 @@ class Application(VuetifyTemplate, HubListener):
             for which there exists data translation functions in the
             glue astronomy repository.
         data_label : str, optional
-            The name associated with this data. If none is given, a generic
-            name is generated.
+            The name associated with this data. If none is given, label is pulled
+            from the input data (if `~glue.core.data.Data`) or a generic name is
+            generated.
+        notify_done: bool
+            Flag controlling whether a snackbar message is set when the data is
+            added to the app. Set to False to avoid overwhelming the user if
+            lots of data is getting loaded at once.
         """
 
-        # Include the data in the data collection
+        if not data_label and hasattr(data, "label"):
+            data_label = data.label
         data_label = data_label or "New Data"
         self.data_collection[data_label] = data
 
         # Send out a toast message
-        snackbar_message = SnackbarMessage(
-            f"Data '{data_label}' successfully added.", sender=self)
-        self.hub.broadcast(snackbar_message)
+        if notify_done:
+            snackbar_message = SnackbarMessage(
+                f"Data '{data_label}' successfully added.", sender=self, color="success")
+            self.hub.broadcast(snackbar_message)
 
     @staticmethod
     def _build_data_label(path, ext=None):
@@ -632,11 +796,14 @@ class Application(VuetifyTemplate, HubListener):
             is required.
         """
         viewer_item = self._viewer_item_by_reference(viewer_reference)
+        if viewer_item is None:  # Maybe they mean the ID
+            viewer_item = self._viewer_item_by_id(viewer_reference)
+        if viewer_item is None:
+            raise ValueError(f"Could not identify viewer with reference {viewer_reference}")
         data_label = self._build_data_label(data_path, ext=ext)
         data_id = self._data_id_from_label(data_label)
 
-        data_ids = viewer_item['selected_data_items'] \
-            if not clear_other_data else []
+        data_ids = viewer_item['selected_data_items'] if not clear_other_data else []
 
         if data_id is not None:
             data_ids.append(data_id)
@@ -707,6 +874,35 @@ class Application(VuetifyTemplate, HubListener):
             if data_item['name'] == label:
                 return data_item['id']
 
+    def get_viewer_ids(self, prefix=None):
+        """Return a list of available viewer IDs.
+
+        Parameters
+        ----------
+        prefix : str or `None`
+            If not `None`, only return viewer IDs with given prefix
+            (case-sensitive). Otherwise, all viewer IDs are returned.
+
+        Returns
+        -------
+        vids : list of str
+            Sorted list of viewer IDs.
+
+        """
+        all_keys = sorted(self._viewer_store.keys())
+
+        if isinstance(prefix, str):
+            vids = [k for k in all_keys if k.startswith(prefix)]
+        else:
+            vids = all_keys
+
+        return vids
+
+    def get_viewer_reference_names(self):
+        """Return a list of available viewer reference names."""
+        # Cannot sort because of None
+        return [self._viewer_item_by_id(vid)['reference'] for vid in self._viewer_store]
+
     def _viewer_by_id(self, vid):
         """
         Viewer instance by id.
@@ -745,7 +941,9 @@ class Application(VuetifyTemplate, HubListener):
                         return viewer_item
 
                 if len(stack_item.get('children')) > 0:
-                    return find_viewer_item(stack_item.get('children'))
+                    result = find_viewer_item(stack_item.get('children'))
+                    if result is not None:
+                        return result
 
         viewer_item = find_viewer_item(self.state.stack_items)
 
@@ -820,6 +1018,14 @@ class Application(VuetifyTemplate, HubListener):
 
         resize(self.state.stack_items)
 
+        # resize tray items
+        for tray_item in self.state.tray_items:
+            # access the actual plugin object (there is no store for plugins)
+            tray_obj = self.widgets.get(tray_item['widget'].split('IPY_MODEL_')[1])
+            for bqplot_fig in tray_obj.bqplot_figs_resize:
+                bqplot_fig.layout.height = '99.9%'
+                bqplot_fig.layout.height = '100%'
+
     def vue_destroy_viewer_item(self, cid):
         """
         Callback for when viewer area tabs are destroyed. Finds the viewer item
@@ -829,7 +1035,7 @@ class Application(VuetifyTemplate, HubListener):
         Parameters
         ----------
         cid : str
-            The UUID associated with the viewer item dictionary.
+            The viewer ID associated with the viewer item dictionary.
         """
         def remove(stack_items):
             for stack in stack_items:
@@ -837,26 +1043,23 @@ class Application(VuetifyTemplate, HubListener):
                     if viewer['id'] == cid:
                         stack['viewers'].remove(viewer)
 
-                # If the stack is empty of viewers, also delete the stack
-                if len(stack['viewers']) == 0 and \
-                        len(stack['children']) == 0:
-                    stack_items.remove(stack)
-                    continue
-
                 if len(stack.get('children', [])) > 0:
                     stack['children'] = remove(stack['children'])
 
+            for empty_stack in [s for s in stack_items
+                                if not s['viewers'] and not s.get('children')]:
+                stack_items.remove(empty_stack)
+
             return stack_items
 
         remove(self.state.stack_items)
 
-        # self.vue_relayout()
-
         # Also remove the viewer from the stored viewer instance dictionary
-        # FIXME: This is getting called twice for some reason
         if cid in self._viewer_store:
             del self._viewer_store[cid]
 
+        self.hub.broadcast(ViewerRemovedMessage(cid, sender=self))
+
     def vue_data_item_selected(self, event):
         """
         Callback for selection events in the front-end data list. Selections
@@ -872,6 +1075,9 @@ class Application(VuetifyTemplate, HubListener):
         viewer_id, item_id, checked = event['id'], event['item_id'], event['checked']
         viewer_item = self._viewer_item_by_id(viewer_id)
 
+        if viewer_item is None:
+            raise ValueError(f'viewer {viewer_id} not found')
+
         if checked:
             selected_items = [*viewer_item['selected_data_items'], item_id]
         else:
@@ -892,6 +1098,19 @@ class Application(VuetifyTemplate, HubListener):
         elif filetype == "svg":
             viewer.figure.save_svg()
 
+    def vue_close_snackbar_message(self, event):
+        """
+        Callback to close a message in the snackbar when the "close"
+        button is clicked.
+        """
+        self.state.snackbar_queue.close_current_message(self.state)
+
+    def vue_call_viewer_method(self, event):
+        viewer_id, method = event['id'], event['method']
+        args = event.get('args', [])
+        kwargs = event.get('kwargs', {})
+        return getattr(self._viewer_store[viewer_id], method)(*args, **kwargs)
+
     def _update_selected_data_items(self, viewer_id, selected_items):
         # Find the active viewer
         viewer_item = self._viewer_item_by_id(viewer_id)
@@ -909,8 +1128,8 @@ class Application(VuetifyTemplate, HubListener):
                           if x['id'] == data_id), None)
 
             if label is None:
-                logging.warning(f"No data item with id '{data_id}' found in "
-                                f"viewer '{viewer_id}'.")
+                warnings.warn(f"No data item with id '{data_id}' found in "
+                              f"viewer '{viewer_id}'.")
                 continue
 
             active_data_labels.append(label)
@@ -942,9 +1161,8 @@ class Application(VuetifyTemplate, HubListener):
         # active data.
         if len(active_data_labels) > 0:
             active_data = self.data_collection[active_data_labels[0]]
-            if hasattr(active_data, "_preferred_translation") \
-                    and active_data._preferred_translation is not None:
-                pass
+            if (hasattr(active_data, "_preferred_translation")
+                    and active_data._preferred_translation is not None):
                 self._set_plot_axes_labels(viewer_id)
 
     def _on_data_added(self, msg):
@@ -955,7 +1173,7 @@ class Application(VuetifyTemplate, HubListener):
 
         Parameters
         ----------
-        msg : `~glue.core.DataCollectionAddMessage`
+        msg : `~glue.core.message.DataCollectionAddMessage`
             The Glue data collection add message containing information about
             the new data.
         """
@@ -970,7 +1188,7 @@ class Application(VuetifyTemplate, HubListener):
 
         Parameters
         ----------
-        msg : `~glue.core.DataCollectionAddMessage`
+        msg : `~glue.core.message.DataCollectionAddMessage`
             The Glue data collection add message containing information about
             the new data.
         """
@@ -1017,8 +1235,17 @@ class Application(VuetifyTemplate, HubListener):
             'children': children,
             'viewers': viewers}
 
-    @staticmethod
-    def _create_viewer_item(viewer, name=None, reference=None):
+    def _next_viewer_num(self, prefix):
+        all_vids = self.get_viewer_ids(prefix=prefix)
+        if len(all_vids) == 0:
+            return 0
+
+        # Assume name-num format
+        last_vid = all_vids[-1]
+        last_num = int(last_vid.split('-')[-1])
+        return last_num + 1
+
+    def _create_viewer_item(self, viewer, vid=None, name=None, reference=None):
         """
         Convenience method for generating viewer item dictionaries.
 
@@ -1026,8 +1253,11 @@ class Application(VuetifyTemplate, HubListener):
         ----------
         viewer : `~glue_jupyter.bqplot.common.BqplotBaseView`
             The ``Bqplot`` viewer instance.
-        name : str, optional
+        vid : str or `None`, optional
+            The ID of the viewer.
+        name : str or `None`, optional
             The name shown in the GoldenLayout tab for this viewer.
+            If `None`, it is the same as viewer ID.
         reference : str, optional
             The reference associated with this viewer as defined in the yaml
             configuration file.
@@ -1041,18 +1271,27 @@ class Application(VuetifyTemplate, HubListener):
         tools.borderless = True
         tools.tile = True
 
+        if vid is None:
+            pfx = self.state.settings.get('configuration', str(name))
+            n = self._next_viewer_num(pfx)
+            vid = f"{pfx}-{n}"
+
         return {
-            'id': str(uuid.uuid4()),
-            'name': name or "Unnamed Viewer",
+            'id': vid,
+            'name': name or vid,
             'widget': "IPY_MODEL_" + viewer.figure_widget.model_id,
             'tools': "IPY_MODEL_" + viewer.toolbar_selection_tools.model_id,
+            'tools_open': False,
             'layer_options': "IPY_MODEL_" + viewer.layer_options.model_id,
             'viewer_options': "IPY_MODEL_" + viewer.viewer_options.model_id,
+            'layer_viewer_open': False,
             'selected_data_items': [],
+            'config': self.config,  # give viewer access to app config/layout
+            'data_open': False,
             'collapse': True,
             'reference': reference}
 
-    def _on_new_viewer(self, msg):
+    def _on_new_viewer(self, msg, vid=None, name=None):
         """
         Callback for when the `~jdaviz.core.events.NewViewerMessage` message is
         raised. This method asks the application handler to generate a new
@@ -1063,6 +1302,14 @@ class Application(VuetifyTemplate, HubListener):
         msg : `~jdaviz.core.events.NewViewerMessage`
             The message received from the ``Hub`` broadcast.
 
+        vid : str or `None`
+            ID of the viewer. If `None`, it is auto-generated
+            from configuration settings.
+
+        name : str or `None`
+            Name of the viewer. If `None`, it is auto-generated
+            from class name.
+
         Returns
         -------
         viewer : `~glue_jupyter.bqplot.common.BqplotBaseView`
@@ -1076,8 +1323,9 @@ class Application(VuetifyTemplate, HubListener):
             viewer.state.x_att = x
 
         # Create the viewer item dictionary
-        new_viewer_item = self._create_viewer_item(
-            viewer=viewer)
+        if name is None:
+            name = viewer.__class__.__name__
+        new_viewer_item = self._create_viewer_item(viewer=viewer, vid=vid, name=name)
 
         new_stack_item = self._create_stack_item(
             container='gl-stack',
@@ -1085,15 +1333,16 @@ class Application(VuetifyTemplate, HubListener):
 
         # Store the glupyter viewer object so we can access the add and remove
         #  data methods in the future
-        self._viewer_store[new_viewer_item['id']] = viewer
+        vid = new_viewer_item['id']
+        self._viewer_store[vid] = viewer
 
         # Add viewer locally
         self.state.stack_items.append(new_stack_item)
 
+        self.session.application.viewers.append(viewer)
+
         # Send out a toast message
-        snackbar_message = SnackbarMessage("New viewer successfully created.",
-                                           sender=self)
-        self.hub.broadcast(snackbar_message)
+        self.hub.broadcast(ViewerAddedMessage(vid, sender=self))
 
         return viewer
 
@@ -1132,6 +1381,8 @@ class Application(VuetifyTemplate, HubListener):
 
         # store the loaded config object
         self._loaded_configuration = config
+        # give the vue templates access to the current config/layout
+        self.config = config['settings'].get('configuration', 'unknown')
 
         self.state.settings.update(config.get('settings'))
 
@@ -1178,6 +1429,8 @@ class Application(VuetifyTemplate, HubListener):
                 'widget': "IPY_MODEL_" + tool.model_id
             })
 
+            self._application_handler._tools[name] = tool
+
         for name in config.get('tray', []):
             tray = tray_registry.members.get(name)
             tray_item_instance = tray.get('cls')(app=self)
@@ -1193,11 +1446,12 @@ class Application(VuetifyTemplate, HubListener):
         """ Resets the application state """
         self.state = ApplicationState()
         self.state.add_callback('stack_items', self.vue_relayout)
+        self._application_handler._tools = {}
 
     def get_configuration(self, path=None, section=None):
-        """ Returns a copy of the application configuration
+        """Returns a copy of the application configuration.
 
-        Returns a copy of the configuration specification.  If path
+        Returns a copy of the configuration specification. If path
         is not specified, returns the currently loaded configuration.
 
         Parameters
@@ -1205,11 +1459,12 @@ class Application(VuetifyTemplate, HubListener):
         path : str, optional
             path to the configuration file to be retrieved.
         section : str, optional
-            A section of the configuration to retrieve
+            A section of the configuration to retrieve.
 
         Returns
         -------
-        A configuration specification dictionary
+        cfg : dict
+            A configuration specification dictionary.
 
         """
 
@@ -1219,3 +1474,38 @@ class Application(VuetifyTemplate, HubListener):
 
         cfg = get_configuration(path=path, section=section, config=config)
         return cfg
+
+    def get_tray_item_from_name(self, name):
+        """Return the instance of a tray item for a given name.
+        This is useful for direct programmatic access to Jdaviz plugins
+        registered under tray items.
+
+        Parameters
+        ----------
+        name : str
+            The name used when the plugin was registered to
+            an internal `~jdaviz.core.registries.TrayRegistry`.
+
+        Returns
+        -------
+        tray_item : obj
+            The instance of the plugin registered to tray items.
+
+        Raises
+        ------
+        KeyError
+            Name not found.
+        """
+        from ipywidgets.widgets import widget_serialization
+
+        tray_item = None
+        for item in self.state.tray_items:
+            if item['name'] == name:
+                ipy_model_id = item['widget']
+                tray_item = widget_serialization['from_json'](ipy_model_id, None)
+                break
+
+        if tray_item is None:
+            raise KeyError(f'{name} not found in app.state.tray_items')
+
+        return tray_item
