@@ -1,19 +1,15 @@
-import pathlib
-import re
+import os
 
 from astropy import units as u
 from astropy.nddata import CCDData
 from glue.core.message import (DataCollectionAddMessage,
                                DataCollectionDeleteMessage)
 from traitlets import List, Unicode, Any, Bool, observe
-from spectral_cube import SpectralCube
-from specutils import SpectralRegion
-from regions import RectanglePixelRegion
+from specutils import Spectrum1D, manipulation, analysis
 
 from jdaviz.core.events import SnackbarMessage
 from jdaviz.core.registries import tray_registry
 from jdaviz.core.template_mixin import TemplateMixin
-from jdaviz.utils import load_template
 
 __all__ = ['MomentMap']
 
@@ -24,7 +20,7 @@ u.add_enabled_units([spaxel])
 
 @tray_registry('cubeviz-moment-maps', label="Moment Maps")
 class MomentMap(TemplateMixin):
-    template = load_template("moment_maps.vue", __file__).tag(sync=True)
+    template_file = __file__, "moment_maps.vue"
     n_moment = Any().tag(sync=True)
     dc_items = List([]).tag(sync=True)
     selected_data = Unicode().tag(sync=True)
@@ -35,8 +31,14 @@ class MomentMap(TemplateMixin):
     spectral_min = Any().tag(sync=True)
     spectral_max = Any().tag(sync=True)
     spectral_unit = Unicode().tag(sync=True)
-    spectral_subset_items = List(["None"]).tag(sync=True)
-    selected_subset = Unicode("None").tag(sync=True)
+    spectral_subset_items = List(["Entire Spectrum"]).tag(sync=True)
+    selected_subset = Unicode("Entire Spectrum").tag(sync=True)
+
+    # NOTE: this is currently cubeviz-specific so will need to be updated
+    # to be config-specific if using within other viewer configurations.
+    viewer_to_id = {'Left': 'cubeviz-0', 'Center': 'cubeviz-1', 'Right': 'cubeviz-2'}
+    viewers = List(['None', 'Left', 'Center', 'Right']).tag(sync=True)
+    selected_viewer = Unicode('None').tag(sync=True)
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
@@ -50,7 +52,6 @@ class MomentMap(TemplateMixin):
         self._selected_data = None
         self.n_moment = 0
         self.moment = None
-        self._filename = None
         self.spectral_min = 0.0
         self.spectral_max = 0.0
         self._spectral_subsets = {}
@@ -63,24 +64,24 @@ class MomentMap(TemplateMixin):
                 # Also set the spectral min and max to default to the full range
                 try:
                     self.selected_data = self.dc_items[i]
-                    cube = self._selected_data.get_object(cls=SpectralCube)
+                    cube = self._selected_data.get_object(cls=Spectrum1D, statistic=None)
                     self.spectral_min = cube.spectral_axis[0].value
                     self.spectral_max = cube.spectral_axis[-1].value
                     self.spectral_unit = str(cube.spectral_axis.unit)
                     break
-                # Skip data that can't be returned as a SpectralCube
+                # Skip data that can't be returned as a Spectrum1D
                 except (ValueError, TypeError):
                     continue
 
     def _on_subset_created(self, msg):
-        """Currently unimplemented due to problems with the SubsetCreateMessafe"""
+        """Currently unimplemented due to problems with the SubsetCreateMessage"""
         raise ValueError(msg)
 
     @observe("selected_data")
     def _on_data_selected(self, event):
         self._selected_data = next((x for x in self.data_collection
                                     if x.label == event['new']))
-        cube = self._selected_data.get_object(cls=SpectralCube)
+        cube = self._selected_data.get_object(cls=Spectrum1D, statistic=None)
         # Update spectral bounds and unit if we've switched to another unit
         if str(cube.spectral_axis.unit) != self.spectral_unit:
             self.spectral_min = cube.spectral_axis[0].value
@@ -89,43 +90,28 @@ class MomentMap(TemplateMixin):
 
     @observe("selected_subset")
     def _on_subset_selected(self, event):
-        # If "None" selected, reset based on bounds of selected data
+        # If "Entire Spectrum" selected, reset based on bounds of selected data
         self._selected_subset = self.selected_subset
-        if self._selected_subset == "None":
-            cube = self._selected_data.get_object(cls=SpectralCube)
+        if self._selected_subset == "Entire Spectrum":
+            cube = self._selected_data.get_object(cls=Spectrum1D, statistic=None)
             self.spectral_min = cube.spectral_axis[0].value
             self.spectral_max = cube.spectral_axis[-1].value
         else:
             spec_sub = self._spectral_subsets[self._selected_subset]
-            unit = u.Unit(self.spectral_unit)
-            spec_reg = SpectralRegion.from_center(spec_sub.center.x * unit,
-                                                  spec_sub.width * unit)
-            self.spectral_min = spec_reg.lower.value
-            self.spectral_max = spec_reg.upper.value
-
-    @observe("filename")
-    def _on_filename_changed(self, event):
-        self._filename = self.filename
+            self.spectral_min = spec_sub.lower.value
+            self.spectral_max = spec_sub.upper.value
 
     def vue_list_subsets(self, event):
         """Populate the spectral subset selection dropdown"""
-        temp_subsets = self.app.get_subsets_from_viewer("spectrum-viewer")
-        temp_list = ["None"]
-        temp_dict = {}
-        # Attempt to filter out spatial subsets
-        for key, region in temp_subsets.items():
-            if type(region) == RectanglePixelRegion:
-                temp_dict[key] = region
-                temp_list.append(key)
-        self._spectral_subsets = temp_dict
-        self.spectral_subset_items = temp_list
+        self._spectral_subsets = self.app.get_subsets_from_viewer("spectrum-viewer")
+        self.spectral_subset_items = ["Entire Spectrum"] + sorted(self._spectral_subsets.keys())
 
-    def vue_calculate_moment(self, event):
+    def vue_calculate_moment(self, *args):
         # Retrieve the data cube and slice out desired region, if specified
-        cube = self._selected_data.get_object(cls=SpectralCube)
+        cube = self._selected_data.get_object(cls=Spectrum1D, statistic=None)
         spec_min = float(self.spectral_min) * u.Unit(self.spectral_unit)
         spec_max = float(self.spectral_max) * u.Unit(self.spectral_unit)
-        slab = cube.spectral_slab(spec_min, spec_max)
+        slab = manipulation.spectral_slab(cube, spec_min, spec_max)
 
         # Calculate the moment and convert to CCDData to add to the viewers
         try:
@@ -134,30 +120,29 @@ class MomentMap(TemplateMixin):
                 raise ValueError("Moment must be a positive integer")
         except ValueError:
             raise ValueError("Moment must be a positive integer")
-        self.moment = slab.moment(n_moment)
-
-        moment_ccd = CCDData(self.moment.array, wcs=self.moment.wcs,
-                             unit=self.moment.unit)
+        # Need transpose to align JWST mirror shape. Not sure why.
+        self.moment = CCDData(analysis.moment(slab, order=n_moment).T)
 
         label = "Moment {}: {}".format(n_moment, self._selected_data.label)
-        fname_label = self._selected_data.label.replace("[", "_").replace("]", "_")
+        fname_label = self._selected_data.label.replace("[", "_").replace("]", "")
         self.filename = "moment{}_{}.fits".format(n_moment, fname_label)
-        self.data_collection[label] = moment_ccd
+        self.data_collection[label] = self.moment
         self.moment_available = True
 
         msg = SnackbarMessage("{} added to data collection".format(label),
                               sender=self, color="success")
         self.hub.broadcast(msg)
 
-    def vue_save_as_fits(self, event):
-        self.moment.write(self._filename)
-        # Let the user know where we saved the file (don't need path if user
-        # specified a full filepath
-        if re.search("/", self._filename) is None:
-            wd = pathlib.Path.cwd()
-            full_path = wd / pathlib.Path(self._filename)
-        else:
-            full_path = self._filename
-        msg = SnackbarMessage("Moment map saved to {}".format(str(full_path)),
-                              sender=self, color="success")
-        self.hub.broadcast(msg)
+        if self.selected_viewer != 'None':
+            # replace the contents in the selected viewer with the results from this plugin
+            self.app.add_data_to_viewer(self.viewer_to_id.get(self.selected_viewer),
+                                        label, clear_other_data=True)
+
+    def vue_save_as_fits(self, *args):
+        if self.moment is None or not self.filename:  # pragma: no cover
+            return
+
+        self.moment.write(self.filename)
+        # Let the user know where we saved the file.
+        self.hub.broadcast(SnackbarMessage(
+            f"Moment map saved to {os.path.abspath(self.filename)}", sender=self, color="success"))
