@@ -1,41 +1,52 @@
 import re
 import numpy as np
 
-import astropy.modeling.models as models
 import astropy.units as u
 from astropy.wcs import WCSSUB_SPECTRAL
 from glue.core.message import (SubsetCreateMessage,
                                SubsetDeleteMessage,
                                SubsetUpdateMessage)
-from specutils import Spectrum1D
+from specutils import Spectrum1D, SpectralRegion
 from specutils.utils import QuantityModel
-from traitlets import Bool, Int, List, Unicode
+from traitlets import Any, Bool, List, Unicode, observe
 from glue.core.data import Data
+from glue.core.subset import Subset, RangeSubsetState, OrState, AndState
+from glue.core.link_helpers import LinkSame
 
 from jdaviz.core.events import AddDataMessage, RemoveDataMessage, SnackbarMessage
 from jdaviz.core.registries import tray_registry
 from jdaviz.core.template_mixin import TemplateMixin
-from jdaviz.utils import load_template
-from .fitting_backend import fit_model_to_spectrum
-from .initializers import initialize, model_parameters
+from jdaviz.core.custom_traitlets import IntHandleEmpty
+from jdaviz.configs.default.plugins.model_fitting.fitting_backend import fit_model_to_spectrum
+from jdaviz.configs.default.plugins.model_fitting.initializers import (MODELS,
+                                                                       initialize,
+                                                                       get_model_parameters)
 
 __all__ = ['ModelFitting']
 
-MODELS = {
-     'Const1D': models.Const1D,
-     'Linear1D': models.Linear1D,
-     'Polynomial1D': models.Polynomial1D,
-     'Gaussian1D': models.Gaussian1D,
-     'Voigt1D': models.Voigt1D,
-     'Lorentz1D': models.Lorentz1D
-     }
+
+class _EmptyParam:
+    def __init__(self, value, unit=None):
+        self.value = value
+        self.unit = unit
+        self.quantity = u.Quantity(self.value,
+                                   self.unit if self.unit is not None else u.dimensionless_unscaled)
 
 
 @tray_registry('g-model-fitting', label="Model Fitting")
 class ModelFitting(TemplateMixin):
     dialog = Bool(False).tag(sync=True)
-    template = load_template("model_fitting.vue", __file__).tag(sync=True)
+    template_file = __file__, "model_fitting.vue"
     dc_items = List([]).tag(sync=True)
+    form_valid_data_selection = Bool(False).tag(sync=True)
+    form_valid_model_component = Bool(False).tag(sync=True)
+
+    selected_data = Unicode("").tag(sync=True)
+    spectral_min = Any().tag(sync=True)
+    spectral_max = Any().tag(sync=True)
+    spectral_unit = Unicode().tag(sync=True)
+    spectral_subset_items = List(["Entire Spectrum"]).tag(sync=True)
+    selected_subset = Unicode("Entire Spectrum").tag(sync=True)
 
     model_label = Unicode().tag(sync=True)
     cube_fit = Bool(False).tag(sync=True)
@@ -45,7 +56,17 @@ class ModelFitting(TemplateMixin):
     eq_error = Bool(False).tag(sync=True)
     component_models = List([]).tag(sync=True)
     display_order = Bool(False).tag(sync=True)
-    poly_order = Int(0).tag(sync=True)
+    poly_order = IntHandleEmpty(0).tag(sync=True)
+
+    # add/replace results for "fit"
+    add_replace_results = Bool(True).tag(sync=True)
+
+    # selected_viewer for "apply to cube"
+    # NOTE: this is currently cubeviz-specific so will need to be updated
+    # to be config-specific if using within other viewer configurations.
+    viewer_to_id = {'Left': 'cubeviz-0', 'Center': 'cubeviz-1', 'Right': 'cubeviz-2'}
+    viewers = List(['None', 'Left', 'Center', 'Right']).tag(sync=True)
+    selected_viewer = Unicode('None').tag(sync=True)
 
     available_models = List(list(MODELS.keys())).tag(sync=True)
 
@@ -62,7 +83,9 @@ class ModelFitting(TemplateMixin):
         self._initialized_models = {}
         self._display_order = False
         self.model_label = "Model"
-        self._selected_data_label = None
+        self._spectral_subsets = {}
+        self._window = None
+        self._original_mask = None
         if self.app.state.settings.get("configuration") == "cubeviz":
             self.cube_fit = True
 
@@ -112,16 +135,25 @@ class ModelFitting(TemplateMixin):
         viewer = self.app.get_viewer('spectrum-viewer')
 
         self.dc_items = [layer_state.layer.label
-                         for layer_state in viewer.state.layers]
+                         for layer_state in viewer.state.layers
+                         if (not isinstance(layer_state.layer, Subset)
+                             or not isinstance(layer_state.layer.subset_state,
+                                               (RangeSubsetState, OrState, AndState)))]
 
-    def _param_units(self, param, order=0):
+    def _param_units(self, param, model_type=None):
         """Helper function to handle units that depend on x and y"""
-        y_params = ["amplitude", "amplitude_L", "intercept"]
+        y_params = ["amplitude", "amplitude_L", "intercept", "scale"]
 
         if param == "slope":
             return str(u.Unit(self._units["y"]) / u.Unit(self._units["x"]))
-        elif param == "poly":
+        elif model_type == 'Polynomial1D':
+            # param names are all named cN, where N is the order
+            order = int(float(param[1:]))
             return str(u.Unit(self._units["y"]) / u.Unit(self._units["x"])**order)
+        elif param == "temperature":
+            return str(u.K)
+        elif param == "scale" and model_type == "BlackBody":
+            return str("")
 
         return self._units["y"] if param in y_params else self._units["x"]
 
@@ -129,10 +161,16 @@ class ModelFitting(TemplateMixin):
         """Insert the results of the model fit into the component_models"""
         for m in self.component_models:
             name = m["id"]
-            if len(self.component_models) > 1:
-                m_fit = self._fitted_model[name]
-            else:
+            if hasattr(self._fitted_model, "submodel_names"):
+                if name in self._fitted_model.submodel_names:
+                    m_fit = self._fitted_model[name]
+                else:
+                    continue
+            elif self._fitted_model.name == name:
                 m_fit = self._fitted_model
+            else:
+                # then the component was not in the fitted model
+                continue
             temp_params = []
             for i in range(0, len(m_fit.parameters)):
                 temp_param = [x for x in m["parameters"] if x["name"] ==
@@ -210,7 +248,8 @@ class ModelFitting(TemplateMixin):
         else:
             return False
 
-    def vue_data_selected(self, event):
+    @observe("selected_data")
+    def _selected_data_changed(self, event):
         """
         Callback method for when the user has selected data from the drop down
         in the front-end. It is here that we actually parse and create a new
@@ -225,12 +264,13 @@ class ModelFitting(TemplateMixin):
             label of the data collection object selected by the user.
         """
         selected_spec = self.app.get_data_from_viewer("spectrum-viewer",
-                                                      data_label=event)
-        # Replace NaNs from collapsed SpectralCube in Cubeviz
+                                                      data_label=event['new'])
+        # Replace NaNs from collapsed Spectrum1D in Cubeviz
         # (won't affect calculations because these locations are masked)
         selected_spec.flux[np.isnan(selected_spec.flux)] = 0.0
 
-        self._selected_data_label = event
+        # Save original mask so we can reset after applying a subset mask
+        self._original_mask = selected_spec.mask
 
         if self._units == {}:
             self._units["x"] = str(
@@ -240,6 +280,38 @@ class ModelFitting(TemplateMixin):
 
         self._spectrum1d = selected_spec
 
+        # Also set the spectral min and max to default to the full range
+        self.selected_subset = "Entire Spectrum"
+        # This is no longer needed for 1D but is preserved for now pending
+        # fixes to Cubeviz for multi-subregion subsets
+        self._window = None
+        self.spectral_min = selected_spec.spectral_axis[0].value
+        self.spectral_max = selected_spec.spectral_axis[-1].value
+        self.spectral_unit = str(selected_spec.spectral_axis.unit)
+
+    def vue_list_subsets(self, event):
+        """Populate the spectral subset selection dropdown"""
+        self._spectral_subsets = self.app.get_subsets_from_viewer("spectrum-viewer",
+                                                                  subset_type="spectral")
+        self.spectral_subset_items = ["Entire Spectrum"] + sorted(self._spectral_subsets.keys())
+
+    @observe("selected_subset")
+    def _on_subset_selected(self, event):
+        # If "Entire Spectrum" selected, reset based on bounds of selected data
+        if self.selected_subset == "Entire Spectrum":
+            self._window = None
+            self.spectral_min = self._spectrum1d.spectral_axis[0].value
+            self.spectral_max = self._spectrum1d.spectral_axis[-1].value
+        else:
+            spec_sub = self._spectral_subsets[self.selected_subset]
+            unit = u.Unit(self.spectral_unit)
+            if hasattr(spec_sub, "center"):
+                spreg = SpectralRegion.from_center(spec_sub.center.x * unit,
+                                                   spec_sub.width * unit)
+                self._window = (spreg.lower, spreg.upper)
+                self.spectral_min = spreg.lower.value
+                self.spectral_max = spreg.upper.value
+
     def vue_model_selected(self, event):
         # Add the model selected to the list of models
         self.temp_model = event
@@ -248,27 +320,6 @@ class ModelFitting(TemplateMixin):
         else:
             self.display_order = False
 
-    def _initialize_polynomial(self, new_model):
-        initialized_model = initialize(
-            MODELS[self.temp_model](name=self.temp_name, degree=self.poly_order),
-            self._spectrum1d.spectral_axis,
-            self._spectrum1d.flux)
-
-        self._initialized_models[self.temp_name] = initialized_model
-        new_model["order"] = self.poly_order
-
-        for i in range(self.poly_order + 1):
-            param = "c{}".format(i)
-            initial_val = getattr(initialized_model, param).value
-            new_model["parameters"].append({"name": param,
-                                            "value": initial_val,
-                                            "unit": self._param_units("poly", i),
-                                            "fixed": False})
-
-        self._update_initialized_parameters()
-
-        return new_model
-
     def _reinitialize_with_fixed(self):
         """
         Reinitialize all component models with current values and the
@@ -278,51 +329,86 @@ class ModelFitting(TemplateMixin):
         temp_models = []
         for m in self.component_models:
             fixed = {}
+
+            # Set the initial values as quantities to make sure model units
+            # are set correctly.
+            initial_values = {p["name"]: u.Quantity(p["value"], p["unit"]) for p in m["parameters"]}
+
             for p in m["parameters"]:
                 fixed[p["name"]] = p["fixed"]
+
             # Have to initialize with fixed dictionary
-            if m["model_type"] == "Polynomial1D":
-                temp_model = MODELS[m["model_type"]](name=m["id"],
-                                                     degree=m["order"],
-                                                     fixed=fixed)
-            else:
-                temp_model = MODELS[m["model_type"]](name=m["id"], fixed=fixed)
-            # Now we can set the parameter values
-            for p in m["parameters"]:
-                setattr(temp_model, p["name"], p["value"])
+            temp_model = MODELS[m["model_type"]](name=m["id"], fixed=fixed,
+                                                 **initial_values, **m.get("model_kwargs", {}))
+
             temp_models.append(temp_model)
+
         return temp_models
 
     def vue_add_model(self, event):
         """Add the selected model and input string ID to the list of models"""
+        # validate provided label (only allow "word characters").   These should already be
+        # stripped by JS in the UI element, but we'll confirm here (especially if this is ever
+        # extended to have better API-support)
+        if re.search(r'\W+', self.temp_name):
+            raise ValueError(f"invalid model component ID {self.temp_name}")
+
+        if self.temp_name in [cm['id'] for cm in self.component_models]:
+            raise ValueError(f"model component ID {self.temp_name} already in use")
+
         new_model = {"id": self.temp_name, "model_type": self.temp_model,
-                     "parameters": []}
+                     "parameters": [], "model_kwargs": {}}
+        model_cls = MODELS[self.temp_model]
 
-        # Need to do things differently for polynomials, since the order varies
         if self.temp_model == "Polynomial1D":
-            new_model = self._initialize_polynomial(new_model)
-        else:
-            # Have a separate private dict with the initialized models, since
-            # they don't play well with JSON for widget interaction
-            initialized_model = initialize(
-                MODELS[self.temp_model](name=self.temp_name),
-                self._spectrum1d.spectral_axis,
-                self._spectrum1d.flux)
+            # self.poly_order is the value in the widget for creating
+            # the new model component.  We need to store that with the
+            # model itself as the value could change for another component.
+            new_model["model_kwargs"] = {"degree": self.poly_order}
+        elif self.temp_model == "BlackBody":
+            new_model["model_kwargs"] = {"output_units": self._units["y"],
+                                         "bounds": {"scale": (0.0, None)}}
 
-            self._initialized_models[self.temp_name] = initialized_model
+        initial_values = {}
+        for param_name in get_model_parameters(model_cls, new_model["model_kwargs"]):
+            # access the default value from the model class itself
+            default_param = getattr(model_cls, param_name, _EmptyParam(0))
+            default_units = self._param_units(param_name,
+                                              model_type=new_model["model_type"])
 
-            for param in model_parameters[new_model["model_type"]]:
-                initial_val = getattr(initialized_model, param).value
-                new_model["parameters"].append({"name": param,
-                                                "value": initial_val,
-                                                "unit": self._param_units(param),
-                                                "fixed": False})
+            if default_param.unit is None:
+                # then the model parameter accepts unitless, but we want
+                # to pass with appropriate default units
+                initial_val = u.Quantity(default_param.value, default_units)
+            else:
+                # then the model parameter has default units.  We want to pass
+                # with jdaviz default units (based on x/y units) but need to
+                # convert the default parameter unit to these units
+                initial_val = default_param.quantity.to(default_units)
+
+            initial_values[param_name] = initial_val
+
+        initialized_model = initialize(
+            MODELS[self.temp_model](name=self.temp_name,
+                                    **initial_values,
+                                    **new_model.get("model_kwargs", {})),
+            self._spectrum1d.spectral_axis,
+            self._spectrum1d.flux)
+
+        # need to loop over parameters again as the initializer may have overridden
+        # the original default value
+        for param_name in get_model_parameters(model_cls, new_model["model_kwargs"]):
+            param_quant = getattr(initialized_model, param_name)
+            new_model["parameters"].append({"name": param_name,
+                                            "value": param_quant.value,
+                                            "unit": str(param_quant.unit),
+                                            "fixed": False})
+
+        self._initialized_models[self.temp_name] = initialized_model
 
         new_model["Initialized"] = True
         self.component_models = self.component_models + [new_model]
 
-        self._update_initialized_parameters()
-
     def vue_remove_model(self, event):
         self.component_models = [x for x in self.component_models
                                  if x["id"] != event]
@@ -343,6 +429,15 @@ class ModelFitting(TemplateMixin):
             return
         models_to_fit = self._reinitialize_with_fixed()
 
+        # Apply mask from selected subset
+        if self.selected_subset != "Entire Spectrum":
+            subset_mask = self.app.get_data_from_viewer("spectrum-viewer",
+                                        data_label = self.selected_subset).mask # noqa
+            if self._spectrum1d.mask is None:
+                self._spectrum1d.mask = subset_mask
+            else:
+                self._spectrum1d.mask += subset_mask
+
         try:
             fitted_model, fitted_spectrum = fit_model_to_spectrum(
                 self._spectrum1d,
@@ -366,17 +461,28 @@ class ModelFitting(TemplateMixin):
         else:
             self._update_parameters_from_fit()
 
+        # Also update the _initialized_models so we can use these values
+        # as the starting point for cube fitting
+        self._update_initialized_parameters()
+
+        # Reset the data mask in case we use a different subset next time
+        self._spectrum1d.mask = self._original_mask
+
     def vue_fit_model_to_cube(self, *args, **kwargs):
 
         if self._warn_if_no_equation():
             return
-        data = self.app.data_collection[self._selected_data_label]
+
+        if self.selected_data in self.app.data_collection.labels:
+            data = self.app.data_collection[self.selected_data]
+        else:  # User selected some subset from spectrum viewer, just use original cube
+            data = self.app.data_collection[0]
 
         # First, ensure that the selected data is cube-like. It is possible
         # that the user has selected a pre-existing 1d data object.
         if data.ndim != 3:
             snackbar_message = SnackbarMessage(
-                f"Selected data {self._selected_data_label} is not cube-like",
+                f"Selected data {self.selected_data} is not cube-like",
                 color='error',
                 sender=self)
             self.hub.broadcast(snackbar_message)
@@ -403,15 +509,21 @@ class ModelFitting(TemplateMixin):
         self.hub.broadcast(snackbar_message)
 
         # Retrieve copy of the models with proper "fixed" dictionaries
-        # TODO: figure out why this was causing the parallel fitting to fail
-        # models_to_fit = self._reinitialize_with_fixed()
-        models_to_fit = self._initialized_models.values()
+        models_to_fit = self._reinitialize_with_fixed()
 
-        fitted_model, fitted_spectrum = fit_model_to_spectrum(
-            spec,
-            models_to_fit,
-            self.model_equation,
-            run_fitter=True)
+        try:
+            fitted_model, fitted_spectrum = fit_model_to_spectrum(
+                spec,
+                models_to_fit,
+                self.model_equation,
+                run_fitter=True,
+                window=self._window)
+        except ValueError:
+            snackbar_message = SnackbarMessage(
+                "Cube fitting failed",
+                color='error', loading=False, sender=self)
+            self.hub.broadcast(snackbar_message)
+            raise
 
         # Save fitted 3D model in a way that the cubeviz
         # helper can access it.
@@ -435,7 +547,11 @@ class ModelFitting(TemplateMixin):
             fitted_spectrum.flux.unit.to_string()
 
         # Add to data collection
-        self.app.data_collection.append(output_cube)
+        self.app.add_data(output_cube, label)
+        if self.selected_viewer != 'None':
+            # replace the contents in the selected viewer with the results from this plugin
+            self.app.add_data_to_viewer(self.viewer_to_id.get(self.selected_viewer),
+                                        label, clear_other_data=True)
 
         snackbar_message = SnackbarMessage(
             "Finished cube fitting",
@@ -459,7 +575,8 @@ class ModelFitting(TemplateMixin):
         else:
             model, spectrum = fit_model_to_spectrum(self._spectrum1d,
                                                     self._initialized_models.values(),
-                                                    self.model_equation)
+                                                    self.model_equation,
+                                                    window=self._window)
 
         self.n_models += 1
         label = self.model_label
@@ -467,6 +584,14 @@ class ModelFitting(TemplateMixin):
             self.app.remove_data_from_viewer('spectrum-viewer', label)
             # Remove the actual Glue data object from the data_collection
             self.data_collection.remove(self.data_collection[label])
-        self.data_collection[label] = spectrum
 
-        self.app.add_data_to_viewer('spectrum-viewer', label)
+        self.app.add_data(spectrum, label)
+
+        # Make sure we link the result spectrum to the data we're fitting
+        data_fitted = self.app.session.data_collection[self.selected_data]
+        data_id = data_fitted.world_component_ids[0]
+        model_id = self.app.session.data_collection[label].world_component_ids[0]
+        self.app.session.data_collection.add_link(LinkSame(data_id, model_id))
+
+        if self.add_replace_results:
+            self.app.add_data_to_viewer('spectrum-viewer', label)
