@@ -1,21 +1,34 @@
-import logging
+import warnings
 
-import astropy.units as u
+from astropy import units as u
 from specutils import SpectralRegion, Spectrum1D
 
 from jdaviz.core.helpers import ConfigHelper
 from jdaviz.core.events import RedshiftMessage
-from ..default.plugins.line_lists.line_list_mixin import LineListMixin
+from jdaviz.configs.default.plugins.line_lists.line_list_mixin import LineListMixin
+
+__all__ = ['Specviz', 'SpecViz']
 
 
-class SpecViz(ConfigHelper, LineListMixin):
-    """
-    SpecViz Helper class
+def _apply_redshift_to_spectra(spectra, redshift):
 
-    """
+    flux = spectra.flux
+    # This is a hack around inability to input separate redshift with
+    # a SpectralAxis instance in Spectrum1D
+    spaxis = spectra.spectral_axis.value * spectra.spectral_axis.unit
+    mask = spectra.mask
+    uncertainty = spectra.uncertainty
+    output_spectra = Spectrum1D(flux, spectral_axis=spaxis,
+                                redshift=redshift, mask=mask,
+                                uncertainty=uncertainty)
+
+    return output_spectra
+
+
+class Specviz(ConfigHelper, LineListMixin):
+    """Specviz Helper class."""
 
     _default_configuration = "specviz"
-    _redshift = 0
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
@@ -44,20 +57,13 @@ class SpecViz(ConfigHelper, LineListMixin):
             if data_label is not None:
                 spectra = {data_label: spectra}
             for key in spectra.keys():
-                flux = spectra[key].flux
-                # This is a hack around inability to input separate redshift with
-                # a SpectralAxis instance in Spectrum1D
-                spaxis = spectra[key].spectral_axis.value * spectra[key].spectral_axis.unit
-                mask = spectra[key].mask
-                uncertainty = spectra[key].uncertainty
-                output_spectra[key] = Spectrum1D(flux, spectral_axis=spaxis,
-                                                 redshift=self._redshift, mask=mask,
-                                                 uncertainty=uncertainty)
+                output_spectra[key] = _apply_redshift_to_spectra(spectra[key], self._redshift)
+
             if apply_slider_redshift == "Warn":
-                logging.warning("Warning: Applying the value from the redshift "
-                                "slider to the output spectra. To avoid seeing this "
-                                "warning, explicitly set the apply_slider_redshift "
-                                "argument to True or False.")
+                warnings.warn("Applying the value from the redshift "
+                              "slider to the output spectra. To avoid seeing this "
+                              "warning, explicitly set the apply_slider_redshift "
+                              "argument to True or False.")
 
             if data_label is not None:
                 output_spectra = output_spectra[data_label]
@@ -66,8 +72,8 @@ class SpecViz(ConfigHelper, LineListMixin):
 
     def get_spectral_regions(self):
         """
-        Retrieves glue subset objects from the spectrum viewer and converts
-        them to `~specutils.SpectralRegion` objects.
+        A simple wrapper around the app-level call to retrieve only spectral
+        subsets, which are now returned as SpectralRegions by default.
 
         Returns
         -------
@@ -75,18 +81,7 @@ class SpecViz(ConfigHelper, LineListMixin):
             Mapping from the names of the subsets to the subsets expressed
             as `specutils.SpectralRegion` objects.
         """
-        regions = self.app.get_subsets_from_viewer("spectrum-viewer")
-
-        spec_regs = {}
-
-        for name, reg in regions.items():
-            unit = reg.meta.get("spectral_axis_unit", u.Unit("Angstrom"))
-            spec_reg = SpectralRegion.from_center(reg.center.x * unit,
-                                                  reg.width * unit)
-
-            spec_regs[name] = spec_reg
-
-        return spec_regs
+        return self.app.get_subsets_from_viewer("spectrum-viewer", subset_type="spectral")
 
     def x_limits(self, x_min=None, x_max=None):
         """Sets the limits of the x-axis
@@ -148,7 +143,7 @@ class SpecViz(ConfigHelper, LineListMixin):
             elif isinstance(min_val, u.Quantity):
                 # Convert user's value to axis' units
                 min_val = min_val.to(axis.unit).value
-            # If auto, set to min_valimum wavelength value
+            # If auto, set to min axis wavelength value
             elif min_val == "auto":
                 min_val = min(axis).value
 
@@ -158,7 +153,7 @@ class SpecViz(ConfigHelper, LineListMixin):
             if isinstance(max_val, u.Quantity):
                 # Convert user's value to axis' units
                 max_val = max_val.to(axis.unit).value
-            # If auto, set to max_valimum wavelength value
+            # If auto, set to max axis wavelength value
             elif max_val == "auto":
                 max_val = max(axis).value
 
@@ -193,31 +188,30 @@ class SpecViz(ConfigHelper, LineListMixin):
     def show(self):
         self.app
 
-    def set_redshift_slider_bounds(self, lower=None, upper=None, step=None):
-        '''
-        Set the upper, lower, or both bounds of the redshift slider. Note
-        that this does not do any sanity checks on the numbers provided based
-        on whether the slider is set to Redshift or Radial Velocity.
-        '''
-        if lower is not None:
-            msg = RedshiftMessage("slider_min", lower, sender=self)
-            self.app.hub.broadcast(msg)
-        if upper is not None:
-            msg = RedshiftMessage("slider_max", upper, sender=self)
-            self.app.hub.broadcast(msg)
-        if step is not None:
-            msg = RedshiftMessage("slider_step", step, sender=self)
-            self.app.hub.broadcast(msg)
+    def set_spectrum_tick_format(self, fmt, axis=None):
+        """
+        Manually set the tick format of one of the axes of the profile viewer.
 
-    def set_redshift(self, new_redshift):
-        '''
-        Apply a redshift to any loaded spectral lines and data. Also updates
-        the value shown in the slider.
-        '''
-        msg = RedshiftMessage("redshift", new_redshift, sender=self)
-        self.app.hub.broadcast(msg)
+        Parameters
+        ----------
+        fmt : str
+            Format of tick marks in the spectrum viewer.
+            For example, ``'0.1e'`` to set scientific notation or ``'0.2f'`` to turn it off.
+        axis : {0, 1}
+            The spectrum viewer data axis.
+            Axis 1 corresponds to the Y-axis and 0 to the X-axis.
 
-    def _redshift_listener(self, msg):
-        '''Save new redshifts (including from the helper itself)'''
-        if msg.param == "redshift":
-            self._redshift = msg.value
+        """
+        if axis not in [0, 1]:
+            warnings.warn("Please use either 0 or 1 for the axis value")
+            return
+
+        # Examples of values for fmt are '0.1e' or '0.2f'
+        self.app.get_viewer("spectrum-viewer").figure.axes[axis].tick_format = fmt
+
+
+# TODO: Officially deprecate this with coordination with JDAT notebooks team.
+# For backward compatibility only.
+class SpecViz(Specviz):
+    """This class is pending deprecation. Please use `Specviz` instead."""
+    pass
