@@ -1,14 +1,15 @@
 from astropy import units as u
 from astropy.nddata import VarianceUncertainty, StdDevUncertainty, InverseVariance
-import numpy as np
-
+from glue.core.message import SubsetCreateMessage, SubsetDeleteMessage
 from specutils import Spectrum1D
 from traitlets import List, Unicode, Any
 
-from jdaviz.core.events import SnackbarMessage, RemoveDataMessage, AddDataMessage
+from jdaviz.core.events import SnackbarMessage, RemoveDataMessage, AddDataMessage, RedshiftMessage
 from jdaviz.core.registries import tray_registry
-from jdaviz.core.template_mixin import TemplateMixin
-from jdaviz.utils import load_template
+from jdaviz.core.template_mixin import PluginTemplateMixin
+from jdaviz.core.validunits import (create_spectral_equivalencies_list,
+                                    create_flux_equivalencies_list)
+from jdaviz.configs.specviz.helper import _apply_redshift_to_spectra
 
 __all__ = ['UnitConversion']
 
@@ -18,9 +19,9 @@ unit_exponents = {StdDevUncertainty: 1,
 
 
 @tray_registry('g-unit-conversion', label="Unit Conversion")
-class UnitConversion(TemplateMixin):
+class UnitConversion(PluginTemplateMixin):
 
-    template = load_template("unit_conversion.vue", __file__).tag(sync=True)
+    template_file = __file__, "unit_conversion.vue"
 
     dc_items = List([]).tag(sync=True)
     selected_data = Unicode().tag(sync=True)
@@ -40,16 +41,18 @@ class UnitConversion(TemplateMixin):
 
         self.spectrum = None
 
-        self.hub.subscribe(self, AddDataMessage,
-                           handler=self._on_viewer_data_changed)
+        self.hub.subscribe(self, AddDataMessage, handler=self._on_viewer_data_changed)
+        self.hub.subscribe(self, RemoveDataMessage, handler=self._on_viewer_data_changed)
+        self.hub.subscribe(self, SubsetCreateMessage, handler=self._on_viewer_subset_changed)
+        self.hub.subscribe(self, SubsetDeleteMessage, handler=self._on_viewer_subset_changed)
 
-        self.hub.subscribe(self, RemoveDataMessage,
-                           handler=self._on_viewer_data_changed)
+        self._redshift = None
+        self.app.hub.subscribe(self, RedshiftMessage, handler=self._redshift_listener)
 
     def _on_viewer_data_changed(self, msg=None):
         """
         Callback method for when data is added or removed from a viewer, or
-        when a subset is created, deleted, or updated. This method receieves
+        when a subset is created, deleted, or updated. This method receives
         a glue message containing viewer information in the case of the former
         set of events, and updates the available data list displayed to the
         user.
@@ -74,7 +77,8 @@ class UnitConversion(TemplateMixin):
         if msg is not None and msg.viewer_id != self._viewer_id:
             return
 
-        self._viewer_data = self.app.get_data_from_viewer('spectrum-viewer')
+        self._viewer_data = self.app.get_data_from_viewer('spectrum-viewer',
+                                                          include_subsets=False)
 
         self.dc_items = [data.label
                          for data in self.app.data_collection
@@ -89,16 +93,27 @@ class UnitConversion(TemplateMixin):
 
         self.update_ui()
 
+    def _on_viewer_subset_changed(self, *args):
+        if len(self.app.data_collection.subset_groups) == 0:
+            self.disabled_msg = ''
+        else:
+            self.disabled_msg = 'Please create Subsets only after unit conversion'
+
+    def _redshift_listener(self, msg):
+        '''Save new redshifts (including from the helper itself)'''
+        if msg.param == "redshift":
+            self._redshift = msg.value
+
     def set_spectrum(self, spectrum, label):
         """
         Set spectrum for unit conversion.
 
         Parameters
-        spectrum : `Spectrum1D`
-            Spectrum for unit conversion.
-        label : `string`
-            Label used to represent spectrum.
         ----------
+        spectrum : `~specutils.Spectrum1D`
+            Spectrum for unit conversion.
+        label : str
+            Label used to represent spectrum.
         """
         self.spectrum = spectrum
         self.selected_data = label
@@ -124,16 +139,21 @@ class UnitConversion(TemplateMixin):
         self.current_spectral_axis_unit = self.spectrum.spectral_axis.unit.to_string()
 
         # Populate drop down with all valid options for unit conversion.
-        self.spectral_axis_unit_equivalencies = self.create_spectral_equivalencies_list()
-        self.flux_unit_equivalencies = self.create_flux_equivalencies_list()
+        self.spectral_axis_unit_equivalencies = create_spectral_equivalencies_list(self.spectrum)
+        self.flux_unit_equivalencies = create_flux_equivalencies_list(self.spectrum)
 
     def vue_unit_conversion(self, *args, **kwargs):
         """
-        Runs when the `apply` button is hit. Tries to change units if `new` units are set
+        Runs when the ``apply`` button is hit. Tries to change units if ``new`` units are set
         and are valid.
         """
+        if self._redshift is not None:
+            # apply the global redshift to the new spectrum
+            spectrum = _apply_redshift_to_spectra(self.spectrum, self._redshift)
+        else:
+            spectrum = self.spectrum
 
-        converted_spec = self.process_unit_conversion(self.spectrum,
+        converted_spec = self.process_unit_conversion(spectrum,
                                                       self.new_flux_unit,
                                                       self.new_spectral_axis_unit)
 
@@ -207,16 +227,16 @@ class UnitConversion(TemplateMixin):
 
         Parameters
         ----------
-        spectrum : `Spectrum1D`
+        spectrum : `specutils.Spectrum1D`
             The spectrum that will have its units converted.
-        new_flux : ``
+        new_flux
             The flux of spectrum will be converted to these units if they are provided.
-        new_spectral_axis : ``
+        new_spectral_axis
             The spectral_axis of spectrum will be converted to these units if they are provided.
 
         Returns
         -------
-        converted_spectrum : `Spectrum1D`
+        converted_spectrum : `specutils.Spectrum1D`
             A new spectrum with converted units.
         """
         set_spectral_axis_unit = spectrum.spectral_axis
@@ -293,109 +313,11 @@ class UnitConversion(TemplateMixin):
 
         # Create new spectrum with new units.
         converted_spectrum = spectrum._copy(flux=set_flux_unit,
+                                            wcs=None,
                                             spectral_axis=set_spectral_axis_unit,
                                             unit=set_flux_unit.unit,
                                             uncertainty=temp_uncertainty
                                             )
+
+        converted_spectrum.redshift = spectrum.redshift
         return converted_spectrum
-
-    def create_spectral_equivalencies_list(self):
-        """
-        Gets all possible conversions from current spectral_axis_unit.
-        """
-        # Get unit equivalencies.
-        curr_spectral_axis_unit_equivalencies = u.Unit(
-            self.spectrum.spectral_axis.unit).find_equivalent_units(
-            equivalencies=u.spectral())
-
-        # Get local units.
-        local_units = [u.Unit(unit) for unit in self._locally_defined_spectral_axis_units()]
-
-        # Remove overlap units.
-        curr_spectral_axis_unit_equivalencies = list(set(curr_spectral_axis_unit_equivalencies)
-                                                     - set(local_units))
-
-        # Convert equivalencies into readable versions of the units and sorted alphabetically.
-        spectral_axis_unit_equivalencies_titles = sorted(self.convert_units_to_strings(
-            curr_spectral_axis_unit_equivalencies))
-
-        # Concatenate both lists with the local units coming first.
-        spectral_axis_unit_equivalencies_titles = sorted(self.convert_units_to_strings(
-            local_units)) + spectral_axis_unit_equivalencies_titles
-
-        return spectral_axis_unit_equivalencies_titles
-
-    def create_flux_equivalencies_list(self):
-        """
-        Gets all possible conversions for flux from current flux units.
-        """
-        # Get unit equivalencies.
-        curr_flux_unit_equivalencies = u.Unit(
-            self.spectrum.flux.unit).find_equivalent_units(
-            equivalencies=u.spectral_density(np.sum(self.spectrum.spectral_axis)),
-                                             include_prefix_units=False) # noqa
-
-        # Get local units.
-        local_units = [u.Unit(unit) for unit in self._locally_defined_flux_units()]
-
-        # Remove overlap units.
-        curr_flux_unit_equivalencies = list(set(curr_flux_unit_equivalencies)
-                                            - set(local_units))
-
-        # Convert equivalencies into readable versions of the units and sort them alphabetically.
-        to_str_func = self.convert_units_to_strings
-        flux_unit_equivalencies_titles = sorted(to_str_func(curr_flux_unit_equivalencies))
-
-        # Concatenate both lists with the local units coming first.
-        flux_unit_equivalencies_titles = sorted(to_str_func(local_units)) + \
-                                         flux_unit_equivalencies_titles # noqa
-
-        return flux_unit_equivalencies_titles
-
-    @staticmethod
-    def _locally_defined_flux_units():
-        """
-        list of defined spectral flux density units.
-        """
-        units = ['Jy', 'mJy', 'uJy',
-                 'W / (m2 Hz)',
-                 'eV / (s m2 Hz)',
-                 'erg / (s cm2)',
-                 'erg / (s cm2 um)',
-                 'erg / (s cm2 Angstrom)',
-                 'erg / (s cm2 Hz)',
-                 'ph / (s cm2 um)',
-                 'ph / (s cm2 Angstrom)',
-                 'ph / (s cm2 Hz)']
-        return units
-
-    @staticmethod
-    def _locally_defined_spectral_axis_units():
-        """
-        list of defined spectral flux density units.
-        """
-        units = ['angstrom', 'nanometer',
-                 'micron', 'hertz', 'erg']
-        return units
-
-    def convert_units_to_strings(self, unit_list):
-        """
-        Convert equivalencies into readable versions of the units.
-
-        Parameters
-        ----------
-        unit_list : list
-            List of either `astropy.units` or strings that can be converted
-            to `astropy.units`.
-
-        Returns
-        -------
-        list
-            A list of the units with their best (i.e., most readable) string version.
-        """
-        return [u.Unit(unit).name
-                if u.Unit(unit) == u.Unit("Angstrom")
-                else u.Unit(unit).long_names[0] if (
-                    hasattr(u.Unit(unit), "long_names") and len(u.Unit(unit).long_names) > 0)
-                else u.Unit(unit).to_string()
-                for unit in unit_list]
