@@ -6,6 +6,7 @@ from glue.core import BaseData
 from glue.core.subset import Subset
 from glue.config import data_translator
 from glue_jupyter.bqplot.profile import BqplotProfileView
+from glue.core.exceptions import IncompatibleAttribute
 
 from astropy import table
 from specutils import Spectrum1D
@@ -14,16 +15,29 @@ from astropy import units as u
 
 
 from jdaviz.core.registries import viewer_registry
-from jdaviz.core.spectral_line import SpectralLine
+from jdaviz.core.marks import SpectralLine
 from jdaviz.core.linelists import load_preset_linelist, get_available_linelists
+from jdaviz.core.freezable_state import FreezableProfileViewerState
+from jdaviz.configs.default.plugins.viewers import JdavizViewerMixin
 
 __all__ = ['SpecvizProfileView']
 
 
 @viewer_registry("specviz-profile-viewer", label="Profile 1D (Specviz)")
-class SpecvizProfileView(BqplotProfileView):
+class SpecvizProfileView(BqplotProfileView, JdavizViewerMixin):
+    # Whether to inherit tools from glue-jupyter automatically. Set this to
+    # False to have full control here over which tools are shown in case new
+    # ones are added in glue-jupyter in future that we don't want here.
+    inherit_tools = False
+
+    tools = ['bqplot:home',
+             'jdaviz:boxzoom', 'jdaviz:xrangezoom',
+             'bqplot:panzoom', 'bqplot:panzoom_x',
+             'bqplot:panzoom_y', 'bqplot:xrange']
+
     default_class = Spectrum1D
     spectral_lines = None
+    _state_cls = FreezableProfileViewerState
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
@@ -31,6 +45,17 @@ class SpecvizProfileView(BqplotProfileView):
         self.display_uncertainties = False
         self.display_mask = False
 
+    def _on_subset_create(self, msg):
+        for layer in self.state.layers:
+            if layer.layer.label == msg.subset.label:
+                layer.linewidth = 3
+
+    def _on_add_data(self, msg):
+        data_label = msg.data.label
+        for layer in self.state.layers:
+            if "Subset" in layer.layer.label and layer.layer.data.label == data_label:
+                layer.linewidth = 3
+
     def data(self, cls=None):
         # Grab the user's chosen statistic for collapsing data
         if hasattr(self.state, 'function'):
@@ -64,21 +89,28 @@ class SpecvizProfileView(BqplotProfileView):
 
                     if _class is not None:
                         handler, _ = data_translator.get_handler_for(_class)
-                        layer_data = handler.to_object(layer_data,
-                                                       statistic=statistic)
+                        try:
+                            layer_data = handler.to_object(layer_data,
+                                                           statistic=statistic)
+                        except IncompatibleAttribute:
+                            continue
                     data.append(layer_data)
 
         return data
 
+    @property
+    def redshift(self):
+        return self.jdaviz_helper._redshift
+
     def load_line_list(self, line_table, replace=False, return_table=False, show=True):
         # If string, load the named preset list and don't show by default
         # since there might be too many lines
-        if type(line_table) == str:
+        if isinstance(line_table, str):
             self.load_line_list(load_preset_linelist(line_table),
                                 replace=replace, return_table=return_table,
                                 show=False)
             return
-        elif type(line_table) != table.QTable:
+        elif not isinstance(line_table, table.QTable):
             raise TypeError("Line list must be an astropy QTable with\
                             (minimally) 'linename' and 'rest' columns")
         if "linename" not in line_table.columns:
@@ -160,7 +192,7 @@ class SpecvizProfileView(BqplotProfileView):
         """
         fig = self.figure
         if name is None and name_rest is None:
-            fig.marks = [x for x in fig.marks if type(x) != SpectralLine]
+            fig.marks = [x for x in fig.marks if not isinstance(x, SpectralLine)]
             if show_none:
                 self.spectral_lines["show"] = False
         else:
@@ -168,23 +200,23 @@ class SpecvizProfileView(BqplotProfileView):
             # Toggle "show" value in main astropy table. The astropy table
             # machinery only allows updating a single row at a time.
             if name_rest is not None:
-                if type(name_rest) == str:
+                if isinstance(name_rest, str):
                     self.spectral_lines.loc[name_rest]["show"] = False
-                elif type(name_rest) == list:
+                elif isinstance(name_rest, list):
                     for nr in name_rest:
                         self.spectral_lines.loc[nr]["show"] = False
             # Get rid of the marks we no longer want
             for x in fig.marks:
-                if type(x) == SpectralLine:
+                if isinstance(x, SpectralLine):
                     if name is not None:
                         self.spectral_lines.loc[name]["show"] = False
                         if x.name == name:
                             continue
                     else:
-                        if type(name_rest) == str:
+                        if isinstance(name_rest, str):
                             if x.table_index == name_rest:
                                 continue
-                        elif type(name_rest) == list:
+                        elif isinstance(name_rest, list):
                             if x.table_index in name_rest:
                                 continue
                 temp_marks.append(x)
@@ -198,8 +230,8 @@ class SpecvizProfileView(BqplotProfileView):
             self.toolbar.active_tool = None
         return {'x': fig.interaction.x_scale, 'y': fig.interaction.y_scale}
 
-    def plot_spectral_line(self, line, scales=None, plot_units=None, **kwargs):
-        if type(line) == str:
+    def plot_spectral_line(self, line, plot_units=None, **kwargs):
+        if isinstance(line, str):
             # Try the full index first (for backend calls), otherwise name only
             try:
                 line = self.spectral_lines.loc[line]
@@ -207,12 +239,11 @@ class SpecvizProfileView(BqplotProfileView):
                 line = self.spectral_lines.loc["linename", line]
         if plot_units is None:
             plot_units = self.data()[0].spectral_axis.unit
-        if scales is None:
-            scales = self.get_scales()
-        # Calculate observed wavelength based on redshift
-        line_val = line["rest"].to(plot_units).value * (1+line['redshift'])
 
-        line_mark = SpectralLine(line_val, scales, name=line["linename"],
+        line_mark = SpectralLine(self,
+                                 line['rest'].to(plot_units).value,
+                                 self.redshift,
+                                 name=line["linename"],
                                  table_index=line["name_rest"],
                                  colors=[line["colors"]], **kwargs)
 
@@ -229,31 +260,27 @@ class SpecvizProfileView(BqplotProfileView):
         fig = self.figure
         self.erase_spectral_lines(show_none=False)
 
-        scales = self.get_scales()
-
         # Check to see if colors were defined for each line
         if "colors" in self.spectral_lines.columns:
             colors = self.spectral_lines["colors"]
         elif len(colors) != len(self.spectral_lines):
             colors = colors*len(self.spectral_lines)
 
-        # Retrieve redshift from table or plotted spectrum
-        if "redshift" in self.spectral_lines.columns:
-            redshifts = self.spectral_lines["redshift"]
-        else:
-            redshifts = [self.data()[0].redshift]*len(self.spectral_lines)
-
         lines = self.spectral_lines
         plot_units = self.data()[0].spectral_axis.unit
 
-        for i in range(len(lines)):
-            if not lines[i]["show"]:
+        marks = []
+        for line, color in zip(lines, colors):
+            if not line["show"]:
                 continue
-            line_val = lines[i]["rest"].to(plot_units).value * (1+redshifts[i])
-            line = SpectralLine(line_val, scales, name=lines[i]["linename"],
-                                table_index=lines[i]["name_rest"],
-                                colors=[colors[i]], **kwargs)
-            fig.marks = fig.marks + [line]
+            line = SpectralLine(self,
+                                line['rest'].to(plot_units).value,
+                                redshift=self.redshift,
+                                name=line["linename"],
+                                table_index=line["name_rest"],
+                                colors=[color], **kwargs)
+            marks.append(line)
+        fig.marks = fig.marks + marks
 
     def available_linelists(self):
         return get_available_linelists()
@@ -297,27 +324,32 @@ class SpecvizProfileView(BqplotProfileView):
 
         Parameters
         ----------
-        spectrum : :class:`glue.core.Data`
+        spectrum : :class:`glue.core.data.Data`
             Data object with the spectrum.
-        color: color specification
-            Color value for plotting
-        alpha: float
-            Alpha value for plotting
+        color : obj
+            Color value for plotting.
+        alpha : float
+            Alpha value for plotting.
 
         Returns
         -------
-        :bool: True if success, False otherwise.
+        result : bool
+            `True` if successful, `False` otherwise.
         """
         # The base class handles the plotting of the main
         # trace representing the spectrum itself.
         result = super().add_data(data, color, alpha, **layer_state)
 
-        # Index of spectrum trace plotted by the super class. It is
-        # for now, always the last in the array of marks. This will
-        # have to be reworked once multiple spectra can be over-plotted.
-        # The uncert and mask trace pointers are meant to facilitate this
-        # eventual upgrade.
-        self.data_trace_pointer = len(self.figure.marks) - 1
+        # Index of spectrum trace plotted by the super class. It is the
+        # **latest** item in figure.marks that is a Line instance
+        for ind, mark in reversed(list(enumerate(self.figure.marks))):
+            # we'll use __class__.__name__ since other entries (spectral lines,
+            # etc) defined in jdaviz.core.marks inherit from Lines
+            if mark.__class__.__name__ == 'Lines':
+                self.data_trace_pointer = ind
+                break
+        else:  # pragma: no cover
+            raise ValueError("could not find mark for added data")
 
         self.error_trace_pointer = None
         self.mask_trace_pointer = None
@@ -426,8 +458,7 @@ class SpecvizProfileView(BqplotProfileView):
         data = self.data()[0]
 
         # Set axes labels for the spectrum viewer
-
-        spectral_axis_unit_type = data.spectral_axis.unit.physical_type.title()
+        spectral_axis_unit_type = str(data.spectral_axis.unit.physical_type).title()
         # flux_unit_type = data.flux.unit.physical_type.title()
         flux_unit_type = "Flux density"
 
@@ -442,3 +473,6 @@ class SpecvizProfileView(BqplotProfileView):
 
         # Make it so y axis label is not covering tick numbers.
         self.figure.axes[1].label_offset = "-50"
+
+        # # Set Y-axis to scientific notation
+        self.figure.axes[1].tick_format = '0.1e'
