@@ -1,13 +1,15 @@
 from glue.core.message import Message
 
-__all__ = ['NewViewerMessage', 'AddViewerMessage', 'LoadDataMessage',
-           'DataSelectedMessage', 'ViewerSelectedMessage',
-           'RemoveStackMessage', 'SplitStackMessage', 'RemoveItemMessage',
+__all__ = ['NewViewerMessage', 'ViewerAddedMessage', 'ViewerRemovedMessage', 'LoadDataMessage',
            'AddDataMessage', 'SnackbarMessage', 'RemoveDataMessage',
-           'AddLineListMessage']
+           'AddLineListMessage', 'RowLockMessage',
+           'SliceSelectWavelengthMessage', 'SliceSelectSliceMessage',
+           'SliceToolStateMessage',
+           'TableClickMessage']
 
 
 class NewViewerMessage(Message):
+    """Message to trigger viewer creation in the application."""
     def __init__(self, cls, data, x_attr=None, *args, **kwargs):
         super().__init__(*args, **kwargs)
 
@@ -28,15 +30,30 @@ class NewViewerMessage(Message):
         return self._x_attr
 
 
-class AddViewerMessage(Message):
-    def __init__(self, viewer, *args, **kwargs):
+class ViewerAddedMessage(Message):
+    """Unlike `NewViewerMessage`, this should be emitted after a viewer is created."""
+    def __init__(self, viewer_id, *args, **kwargs):
         super().__init__(*args, **kwargs)
 
-        self._viewer = viewer
+        # NOT viewer reference, which is not unique and can be None
+        self._viewer_id = viewer_id
 
     @property
-    def viewer(self):
-        return self._viewer
+    def viewer_id(self):
+        return self._viewer_id
+
+
+class ViewerRemovedMessage(Message):
+    """Message emitted after a viewer is destroyed by the application."""
+    def __init__(self, viewer_id, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+
+        # NOT viewer reference, which is not unique and can be None
+        self._viewer_id = viewer_id
+
+    @property
+    def viewer_id(self):
+        return self._viewer_id
 
 
 class LoadDataMessage(Message):
@@ -50,66 +67,6 @@ class LoadDataMessage(Message):
         return self._path
 
 
-class DataSelectedMessage(Message):
-    def __init__(self, indicies, *args, **kwargs):
-        super().__init__(*args, **kwargs)
-
-        self._indicies = indicies
-
-    @property
-    def indicies(self):
-        return self._indicies
-
-
-class ViewerSelectedMessage(Message):
-    def __init__(self, viewer, *args, **kwargs):
-        super().__init__(*args, **kwargs)
-
-        self._viewer = viewer
-
-    @property
-    def viewer(self):
-        return self._viewer
-
-
-class RemoveStackMessage(Message):
-    def __init__(self, stack, *args, **kwargs):
-        super().__init__(*args, **kwargs)
-
-        self._stack = stack
-
-    @property
-    def stack(self):
-        return self._stack
-
-
-class SplitStackMessage(Message):
-    def __init__(self, item, horizontal, *args, **kwargs):
-        super().__init__(*args, **kwargs)
-
-        self._item = item
-        self._horizontal = horizontal
-
-    @property
-    def item(self):
-        return self._item
-
-    @property
-    def horizontal(self):
-        return self._horizontal
-
-
-class RemoveItemMessage(Message):
-    def __init__(self, item, *args, **kwargs):
-        super().__init__(*args, **kwargs)
-
-        self._item = item
-
-    @property
-    def item(self):
-        return self._item
-
-
 class AddDataMessage(Message):
     def __init__(self, data, viewer, viewer_id=None, *args, **kwargs):
         super().__init__(*args, **kwargs)
@@ -247,3 +204,63 @@ class RedshiftMessage(Message):
     @property
     def value(self):
         return self._value
+
+
+class RowLockMessage(Message):
+    def __init__(self, is_locked, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self._is_locked = is_locked
+
+    @property
+    def is_locked(self):
+        return self._is_locked
+
+
+class TableClickMessage(Message):
+    '''Message generated by Mosviz table to zoom to object on image'''
+    def __init__(self, selected_index, shared_image=False, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self._selected_index = selected_index
+        self._shared_image = shared_image
+
+    @property
+    def selected_index(self):
+        return self._selected_index
+
+    @property
+    def shared_image(self):
+        return self._shared_image
+
+
+class SliceSelectWavelengthMessage(Message):
+    '''Message generated by the select slice plot plugin which is processed by the cubeviz helper'''
+    def __init__(self, wavelength=None, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self._wavelength = wavelength
+
+    @property
+    def wavelength(self):
+        return self._wavelength
+
+
+class SliceSelectSliceMessage(Message):
+    '''Message generated by the cubeviz helper and processed by the slice plugin to sync
+    slice selection across all viewers'''
+    def __init__(self, slice=None, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self._slice = slice
+
+    @property
+    def slice(self):
+        return self._slice
+
+
+class SliceToolStateMessage(Message):
+    '''Message generated by the select slice plot plugin when activated/deactivated'''
+    def __init__(self, change, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self._change = change
+
+    @property
+    def change(self):
+        return self._change
