@@ -1,14 +1,20 @@
 from ipyvuetify import VuetifyTemplate
 from glue.core import HubListener
+from traitlets import Unicode
 
-__all__ = ['TemplateMixin']
+from jdaviz import __version__
+
+__all__ = ['TemplateMixin', 'PluginTemplateMixin']
 
 
 class TemplateMixin(VuetifyTemplate, HubListener):
+    config = Unicode("").tag(sync=True)
+    vdocs = Unicode("").tag(sync=True)
+
     def __new__(cls, *args, **kwargs):
         """
         Overload object creation so that we can inject a reference to the
-        ``Hub`` class before components can be initialized. This makes it so
+        `~glue.core.hub.Hub` class before components can be initialized. This makes it so
         hub references on plugins can be passed along to components in the
         call to the initialization method.
         """
@@ -16,14 +22,23 @@ class TemplateMixin(VuetifyTemplate, HubListener):
         obj = super().__new__(cls, *args, **kwargs)
         obj._app = app
 
+        # give the vue templates access to the current config/layout
+        obj.config = app.state.settings.get("configuration", "default")
+
+        # give the vue templates access to jdaviz version
+        obj.vdocs = 'latest' if 'dev' in __version__ else 'v'+__version__
+
+        # store references to all bqplot widgets that need to handle resizing
+        obj.bqplot_figs_resize = []
+
         return obj
 
     @property
     def app(self):
         """
-        Allows access to the underlying jdaviz application instance. This is
+        Allows access to the underlying Jdaviz application instance. This is
         **not** access to the helper class, but instead the
-        `jdaviz.Application` object.
+        `jdaviz.app.Application` object.
         """
         return self._app
 
@@ -38,3 +53,7 @@ class TemplateMixin(VuetifyTemplate, HubListener):
     @property
     def data_collection(self):
         return self._app.session.data_collection
+
+
+class PluginTemplateMixin(TemplateMixin):
+    disabled_msg = Unicode("").tag(sync=True)
